@@ -41,7 +41,7 @@ from functools import partial
 from pathlib import Path
 from typing import List
 
-__version__ = '1.6.5'
+__version__ = '1.7.1'
 
 _LOGGER = logging.getLogger('tftest')
 
@@ -54,7 +54,10 @@ TerraformStateResource = collections.namedtuple(
 
 
 class TerraformTestError(Exception):
-  pass
+
+  @property
+  def cmd_error(self):
+    return self.args[1] if len(self.args) > 1 else None
 
 
 _TG_BOOL_ARGS = [
@@ -338,13 +341,16 @@ class TerraformTest(object):
               f'Unable to restore terraform file {bkp_file.resolve()}')
           raise TerraformTestError(
               f'Restore of terraform file ({bkp_file.resolve()}) failed')
+        else:
+          bkp_file.unlink(True)
 
   def _abspath(self, path):
     """Make relative path absolute from base dir."""
-    return path if path.startswith('/') else os.path.join(self._basedir, path)
+    return path if os.path.isabs(path) else os.path.join(self._basedir, path)
 
   def setup(self, extra_files=None, plugin_dir=None, init_vars=None,
-            backend=True, cleanup_on_exit=True, disable_prevent_destroy=False, **kw):
+            backend=True, cleanup_on_exit=True, disable_prevent_destroy=False,
+            workspace_name=None, **kw):
     """Setup method to use in test fixtures.
 
     This method prepares a new Terraform environment for testing the module
@@ -359,6 +365,7 @@ class TerraformTest(object):
       backend: Terraform backend argument
       cleanup_on_exit: remove .terraform and terraform.tfstate files on exit
       disable_prevent_destroy: set all prevent destroy to false
+      workspace_name: name of workspace to create or select
 
     Returns:
       Terraform init output.
@@ -382,7 +389,6 @@ class TerraformTest(object):
           with open(tf_file, 'r') as src:
             terraform = src.read()
           with open(tf_file, 'w') as src:
-
             terraform = re.sub(
                 r'prevent_destroy\s+=\s+true', 'prevent_destroy = false', terraform)
             src.write(terraform)
@@ -412,8 +418,13 @@ class TerraformTest(object):
       else:
         _LOGGER.warning('no such file {}'.format(link_src))
     self._finalizer = weakref.finalize(
-        self, self._cleanup, self.tfdir, filenames, deep=cleanup_on_exit, restore_files=disable_prevent_destroy)
-    return self.init(plugin_dir=plugin_dir, init_vars=init_vars, backend=backend, **kw)
+        self, self._cleanup, self.tfdir, filenames, deep=cleanup_on_exit,
+        restore_files=disable_prevent_destroy)
+    setup_output = self.init(plugin_dir=plugin_dir,
+                             init_vars=init_vars, backend=backend, **kw)
+    if workspace_name:
+      setup_output += self.workspace(name=workspace_name)
+    return setup_output
 
   def init(self, input=False, color=False, force_copy=False, plugin_dir=None,
            init_vars=None, backend=True, **kw):
@@ -423,6 +434,14 @@ class TerraformTest(object):
                           init_vars=init_vars, **kw)
     return self.execute_command('init', *cmd_args).out
 
+  def workspace(self, name=None):
+    """Run Terraform workspace command."""
+    raw_ws_out = self.execute_command('workspace', *['list']).out
+    cmd_args = ['select', name]
+    if name not in [ws.replace('*', '').strip() for ws in raw_ws_out.split('\n') if len(ws) > 0]:
+      cmd_args = ['new', name]
+    return self.execute_command('workspace', *cmd_args).out
+
   def plan(self, input=False, color=False, refresh=True, tf_vars=None,
            targets=None, output=False, tf_var_file=None, **kw):
     """
@@ -539,7 +558,6 @@ class TerraformTest(object):
           _LOGGER.info(output.strip())
           full_output_lines.append(output)
       retcode = p.poll()
-      p.stdout.close()
       p.wait()
     except FileNotFoundError as e:
       raise TerraformTestError('Terraform executable not found: %s' % e)
@@ -549,7 +567,7 @@ class TerraformTest(object):
       message = 'Error running command {command}: {retcode} {out} {err}'.format(
           command=cmd, retcode=retcode, out=full_output, err=err)
       _LOGGER.critical(message)
-      raise TerraformTestError(message)
+      raise TerraformTestError(message, err)
     return TerraformCommandOutput(retcode, full_output, err)
 
   def _tg_ra(self) -> List[str]:
